/****************************************************/
/*                                                  */
/*  This Exaple Code demonstates use of following   */
/*  Data Cache control instructions                 */
/*  (a) PREFETCH                                    */
/*  (b) FLUSH                                       */
/*  (c) FLUSHINV                                    */
/*                                                  */
/*                      Initial Version             */
/*                      Kunal Singh                 */
/*                      DSP applications, ADI       */
/*                      Dated 3 April, 2005         */
/*                                                  */
/*                      Re-Release                  */
/*                      for VisualDSP++ 5.0         */
/*                      Andreas Pellkofer           */
/*                      GPDSP Applications, ADI     */
/*                      Dated 9 March, 2009         */
/*                                                  */
/****************************************************/

#include <blackfin.h>
#include <services_types.h>
#include <stdio.h>
#include "main.h"

/************************************************************/
/* Number of entries for the ICPLB and DCPLB descriptors    */
/************************************************************/
#define DCPLB_CNT 10
#define ICPLB_CNT 9
 

/************************************************/
/*       Entries for the DCPLBx_ADDR            */
/************************************************/
#pragma section ("data1")

u32 D_cplb_addrs[DCPLB_CNT] =
    {
        0xff800000,
        0xff900000,
        0x00000000,
        0x00400000,
        0x00800000,
        0x00C00000,
        0x01000000,
        0x01400000,
        0x01800000,
        0x01C00000
    };


/************************************************/
/*      Entries for the DCPLBx_DATA             */
/************************************************/
#pragma section ("data1")

u32 D_cplb_data[DCPLB_CNT] =
    {
        0x2001f,
        0x2001f,
        0x3909f,
        0x3D01f,
        0x3D01f,
        0x3D01f,
        0x3D01f,
        0x3D01f,
        0x3D01f,
        0x3D01f
    };


/************************************************/
/*      Entries for the ICPLBx_ADDR             */
/************************************************/
#pragma section ("data1")

u32 I_cplb_addrs[9] =
    {
        0xFFA00000,
        0x00000000,
        0x00400000,
        0x00800000,
        0x00C00000,
        0x01000000,
        0x01400000,
        0x01800000,
        0x01C00000
    };


/************************************************/
/*      Entries for the ICPLBx_DATA             */
/************************************************/
#pragma section ("data1")

u32 I_cplb_data[9] =
    {
        0x00020003,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005
    };


/********************************/
/*  Lock Control for the ICache */
/*      1: Lock the way         */
/*      0: Don't Lock           */
/********************************/
#pragma section ("data1")

u32 Lock_Control[4] =
    {
        0,
        0,
        0,
        0
    };


#pragma section("sdram0")
u32 data_A [ARRAY_LENGTH];


#pragma section("sdram1")
u32 data_B [ARRAY_LENGTH];


#pragma section("sdram2")
u32 Result [ARRAY_LENGTH];


volatile u32 decLow, decHigh;

u32 cache;

void main()
{
    u32 i;
    u32 dcplb_cnt, icplb_cnt, d_bank_cnt, disable_lru;

    /****************************************************/
    /*  Configure the DCPLBs and Enable Cache           */
    /*  Enabled both the bank as cache                  */
    /*                                                  */
    /*  The page containing address 0x0 is configured   */
    /*  in Write Back Mode                              */
    /****************************************************/
    dcplb_cnt = DCPLB_CNT;
    d_bank_cnt = 2;
    Config_D_Cache(D_cplb_addrs, D_cplb_data, dcplb_cnt, d_bank_cnt);

    /************************************/
    /* Clear the DTEST_DATA0 register   */
    /************************************/
    *pDTEST_DATA0 = 0;
    asm("ssync;");


    /************************************/
    /* Prefetch the Address 0x0         */
    /************************************/
    asm("p0.l = 0;");
    asm("p0.h = 0;");
    asm("PREFETCH [p0];");
    asm("ssync;");
    /****************************************/
    /* Read the Tag to confirm the Prefetch */
    /****************************************/
    *pDTEST_COMMAND = 0;
    asm("ssync;");


    /************************************************/
    /* This write will make the cache line dirty    */
    /************************************************/
    data_A[0] = 3;
    asm("ssync;");


    /********************************************/
    /* Read the Tag to confirm the Dirty Bit    */
    /********************************************/
    *pDTEST_COMMAND = 0;
    asm("ssync;");


    /********************************/
    /* FLUSH the cache line         */
    /* The cache line wil be clean  */
    /********************************/
    asm("p0.l = 0;");
    asm("p0.h = 0;");
    asm("FLUSH [p0];");
    asm("ssync;");


    /************************************/
    /* Read the Tag to confirm FLUSH    */
    /************************************/
    *pDTEST_COMMAND = 0;
    asm("ssync;");


    /************************************************/
    /* This write will make the cache line dirty    */
    /************************************************/
    data_A[0] = 3;
    asm("ssync;");


    /********************************************/
    /* Read the Tag to confirm the Dirty Bit    */
    /********************************************/
    *pDTEST_COMMAND = 0;
    asm("ssync;");


    /************************************/
    /* This FLUSHINV will clean         */
    /* and Invalidate the cache line    */
    /************************************/
    asm("p0.l = 0;");
    asm("p0.h = 0;");
    asm("FLUSHINV [p0];");
    asm("ssync;");


    /************************************/
    /* Read the Tag to confirm FLUSHINV */
    /************************************/
    *pDTEST_COMMAND = 0;
    asm("ssync;");

    printf("done...\n");
}
